# Generated by roxygen2: do not edit by hand

export(calculate_features)
export(init_theft)
export(init_theft_hctsa)
export(init_theft_kats)
export(init_theft_tsfel)
export(init_theft_tsfresh)
export(install_kats)
export(install_pyhctsa)
export(install_python_pkgs)
export(install_tsfel)
export(install_tsfresh)
export(moments)
export(process_hctsa_file)
export(quantiles)
import(dplyr)
import(feasts)
importFrom(R.matlab,readMat)
importFrom(Rcatch22,catch22_all)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,reframe)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,sym)
importFrom(dplyr,ungroup)
importFrom(fabletools,feature_set)
importFrom(fabletools,features)
importFrom(purrr,map_df)
importFrom(reticulate,import)
importFrom(reticulate,source_python)
importFrom(reticulate,use_virtualenv)
importFrom(reticulate,virtualenv_create)
importFrom(reticulate,virtualenv_install)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,ts)
importFrom(stats,var)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,unnest_wider)
importFrom(tsfeatures,tsfeatures)
importFrom(tsibble,as_tsibble)
importFrom(tsibble,index_var)
importFrom(tsibble,key_vars)
importFrom(utils,packageVersion)
